@@ -74,8 +74,11 @@ type interface_state_t = {
    len_untimed       : int;                 (* number of entries in the untimed reminders list *)
    search_regex      : Str.regexp;          (* most recent search string *)
    search_input      : string;              (* buffer to hold search string input *)
+   goto_input        : string;              (* buffer to hold 'go to date' input *)
    is_entering_search: bool;                (* whether or not the user is entering a search string *)
-   last_timed_refresh: float                (* the last time the timed window had a complete refresh *)
+   is_entering_goto  : bool;                (* whether or not the user is entering a date to navigate to *)
+   last_timed_refresh: float;               (* the last time the timed window had a complete refresh *)
+   rem_buffer        : string               (* buffer that acts as a clipboard for REM strings *)
 }
    
 
@@ -122,8 +125,11 @@ let make (std : screen_t) =
       len_untimed        = 0;
       search_regex       = Str.regexp "";
       search_input       = "";
+      goto_input         = "";
       is_entering_search = false;
+      is_entering_goto   = false;
       last_timed_refresh = 0.0;
+      rem_buffer         = ""
    }
                                                
 
