@@ -137,11 +137,11 @@ let resize_subwins iface =
 
 (* refresh the screen *)
 let handle_refresh (iface : interface_state_t) reminders =
-   let _ = touchwin iface.scr.help_win in
-   let _ = touchwin iface.scr.timed_win in
-   let _ = touchwin iface.scr.calendar_win in
-   let _ = touchwin iface.scr.untimed_win in
-   let _ = touchwin iface.scr.msg_win in 
+   let _ = wtouchln iface.scr.help_win 0 1 true in
+   let _ = wtouchln iface.scr.timed_win 0 iface.scr.tw_lines true in
+   let _ = wtouchln iface.scr.calendar_win 0 iface.scr.cw_lines true in
+   let _ = wtouchln iface.scr.untimed_win 0 iface.scr.uw_lines true in
+   let _ = wtouchln iface.scr.msg_win 0 iface.scr.mw_lines true in
    draw_help iface;
    let new_iface = draw_timed iface reminders.Remind.all_timed in
    draw_date_strip new_iface;
@@ -500,6 +500,7 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
        remfile =
    let ts = timestamp_of_line iface iface.left_selection in
    let tm = Unix.localtime ts in
+   (* take care of the substitution characters in remline templates *)
    let substitute template =
       let rec substitute_aux subst_list s =
          match subst_list with
@@ -544,11 +545,13 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
             | (Some t, _)          -> substitute t
             end
       in
+      (* append the remline to the reminders file *)
       let remfile_channel = 
          open_out_gen [Open_append; Open_creat; Open_text] 416 remfile
       in
       output_string remfile_channel remline;
       close_out remfile_channel;
+      (* open the reminders file in an editor *)
       let filename_sub = Str.regexp "%f" in
       let command = 
          Str.global_replace filename_sub remfile
@@ -556,7 +559,7 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
       in
       def_prog_mode ();
       endwin ();
-      let _ = Unix.system command in 
+      let editor_process_status = Unix.system command in 
       reset_prog_mode ();
       begin try
          assert (curs_set 0)
@@ -576,7 +579,24 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
          else
             iface
       in
-      handle_refresh new_iface r
+      begin match editor_process_status with
+      | Unix.WEXITED return_code ->
+         if return_code <> 0 then begin
+            let (new_iface, r)  = handle_refresh new_iface r in
+            let _ = beep () in
+            draw_error new_iface 
+               "Error when launching editor; configure a different editor in ~/.wyrdrc ." false;
+            assert (doupdate ());
+            (new_iface, r)
+         end else
+            handle_refresh new_iface r
+      | _ -> 
+         let (new_iface, r) = handle_refresh new_iface r in
+         draw_error new_iface
+            "Editor process was interrupted." false;
+         assert (doupdate ());
+         (new_iface, r)
+      end
    with Template_undefined template_str ->
       draw_error iface (template_str ^ " is undefined.") false;
       assert (doupdate ());
@@ -1051,9 +1071,335 @@ let handle_scroll_desc_down iface reminders =
    handle_refresh new_iface reminders
 
 
+(* auxiliary function for the following *)
+let handle_copy_reminder_aux iface reminders copy_only =
+   let adjust_s len s =
+      let pad = s ^ (String.make len ' ') in
+      Str.string_before pad len
+   in
+   let fl =
+      match iface.selected_side with
+      |Left  -> 
+         begin match iface.timed_lineinfo.(iface.left_selection) with
+         | [] -> 
+            None
+         | (f, l, t, m, s) :: [] -> 
+            Some (f, l)
+         | rem_list -> 
+              let sorted_rem_list = List.fast_sort sort_lineinfo rem_list in
+              let get_msg (_, _, time, msg, _) = (adjust_s 16 time) ^ msg in
+              let msg_list = List.rev_map get_msg sorted_rem_list in
+              let selected_msg =
+                 let dialog_msg =
+                    if copy_only then
+                       "Choose a reminder to copy"
+                    else
+                       "Choose a reminder to cut"
+                 in
+                 do_selection_dialog iface dialog_msg msg_list
+              in
+              let _ = handle_refresh iface reminders in
+              let test_msg_match (_, _, time, msg, _) = 
+                 ((adjust_s 16 time) ^ msg) = selected_msg in
+              let (f, l, t, m, s) = (List.find test_msg_match sorted_rem_list) in
+              Some (f, l)
+         end
+      |Right ->
+         begin match iface.untimed_lineinfo.(iface.right_selection) with
+         |Some (f, l, m) -> Some (f, l)
+         |None -> None
+         end
+   in
+   begin match fl with
+   |None ->
+      let _ = beep () in
+      draw_error iface "no reminder is selected." false;
+      assert (doupdate ());
+      (iface, fl)
+   |Some (filename, line_num_str) ->
+      let in_channel = open_in filename in
+      let cont_regex = Str.regexp ".*\\(\\\\\\)$" in
+      begin try
+         (* grab a copy of the specified line *)
+         let line = ref "" in
+         for curr_line = 1 to int_of_string line_num_str do
+            (* concatenate lines with trailing backslashes *)
+            if Str.string_match cont_regex !line 0 then
+               let bs_loc = Str.group_beginning 1 in
+               line := (Str.string_before !line bs_loc) ^ (input_line in_channel)
+            else
+               line := input_line in_channel
+         done;
+         close_in in_channel;
+         (* break the REM line up into chunks corresponding to the different fields *)
+         let rem_kw_regex = 
+            Str.regexp ("\\(REM\\|PRIORITY\\|SKIP\\|BEFORE\\|AFTER\\|" ^
+            "OMIT\\|AT\\|SCHED\\|WARN\\|UNTIL\\|SCANFROM\\|DURATION\\|TAG\\|" ^
+            "MSG\\|MSF\\|RUN\\|CAL\\|SATISFY\\|SPECIAL\\|PS\\|PSFILE\\)")
+         in
+         let rem_chunks = Str.full_split rem_kw_regex !line in
+         (* merge the chunks back together, but replace the REM and AT clauses
+          * with special markers *)
+         let rec merge_chunks chunks s found_rem =
+            match chunks with
+            | [] ->
+               if found_rem then s else "REMREPLACE " ^ s
+            | (Str.Delim "REM") :: (Str.Text datespec) :: tail ->
+               merge_chunks tail (s ^ "REMREPLACE") true
+            | (Str.Delim "REM") :: tail ->
+               merge_chunks tail (s ^ "REMREPLACE") true
+            | (Str.Delim "AT")  :: (Str.Text timespec) :: tail ->
+               merge_chunks tail (s ^ "ATREPLACE") found_rem
+            | (Str.Delim "AT")  :: tail ->
+               merge_chunks tail (s ^ "ATREPLACE") found_rem
+            | (Str.Delim x)      :: tail ->
+               merge_chunks tail (s ^ x) found_rem
+            | (Str.Text x)      :: tail ->
+               merge_chunks tail (s ^ x) found_rem
+         in
+         let marked_remline = merge_chunks rem_chunks "" false in
+         let new_iface = {iface with rem_buffer = marked_remline} in
+         if copy_only then begin
+            draw_error iface "copied reminder to clipboard." false;
+            assert (doupdate ());
+         end else
+            ();
+         (new_iface, fl)
+      with
+      |End_of_file ->
+         close_in in_channel;
+         let _ = beep () in
+         draw_error iface "error while copying reminder." false;
+         assert (doupdate ());
+         (iface, fl)
+      |_ ->
+         let _ = beep () in
+         draw_error iface "unable to parse selected reminder." false;
+         assert (doupdate ());
+         (iface, fl)
+      end
+   end
+
+
+(* handle copying a reminder string to the buffer *)
+let handle_copy_reminder iface reminders copy_only =
+   let new_iface, _ = handle_copy_reminder_aux iface reminders copy_only in
+   (new_iface, reminders)
+
+
+(* handle pasting a reminder into a new location *)
+let handle_paste_reminder (iface : interface_state_t) reminders remfile =
+   let ts = timestamp_of_line iface iface.left_selection in
+   let tm = Unix.localtime ts in
+   let rem_datespec = 
+      "REM " ^ Utility.string_of_tm_mon tm.Unix.tm_mon ^ " " ^
+      string_of_int tm.Unix.tm_mday ^ " " ^
+      string_of_int (tm.Unix.tm_year + 1900) ^ " "
+   and at_timespec =
+      "AT " ^ Printf.sprintf "%.2d" tm.Unix.tm_hour ^ ":" ^
+      Printf.sprintf "%.2d" tm.Unix.tm_min ^ " "
+   in
+   (* replace REMREPLACE and ATREPLACE keywords by sensible REM and
+    * AT clauses corresponding to the selected date and time *)
+   let rem_regex = Str.regexp "REMREPLACE" in
+   let at_regex  = Str.regexp "ATREPLACE" in
+   let temp        = Str.replace_first rem_regex rem_datespec iface.rem_buffer in
+   let new_remline = Str.replace_first at_regex at_timespec temp in
+   (* paste into the reminders file *)
+   let remfile_channel = 
+      open_out_gen [Open_append; Open_creat; Open_text] 416 remfile
+   in
+   output_string remfile_channel new_remline;
+   close_out remfile_channel;
+   (* open reminders file in editor *)
+   let filename_sub = Str.regexp "%f" in
+   let command = 
+      Str.global_replace filename_sub remfile
+         !Rcfile.edit_new_command
+   in
+   def_prog_mode ();
+   endwin ();
+   let _ = Unix.system command in 
+   reset_prog_mode ();
+   begin try
+      assert (curs_set 0)
+   with _ ->
+      ()
+   end;
+   let r = Remind.create_three_month iface.top_timestamp in
+   (* if the untimed list has been altered, change the focus to
+    * the first element of the list *)
+   let new_iface =
+      if List.length r.Remind.curr_untimed <> 
+         List.length reminders.Remind.curr_untimed then {
+         iface with top_untimed = 0;
+                    top_desc = 0;
+                    right_selection = 1
+         }
+      else
+         iface
+   in
+   handle_refresh new_iface r
+
+
+(* handle cutting a reminder and dropping it in the clipboard *)
+let handle_cut_reminder iface reminders =
+   (* yes, there's some redundant code here... maybe cut and copy
+    * should be merged into one function? *)
+   let (iface, fl) = handle_copy_reminder_aux iface reminders false in
+   begin match fl with
+   |None ->
+      (iface, reminders)
+   |Some (filename, line_num_str) ->
+      let in_channel = open_in filename in
+      let cont_regex = Str.regexp ".*\\(\\\\\\)$" in
+      (* load in the file, but skip over the selected reminder *)
+      let rec process_in_lines lines continuations line_num =
+         try
+            let line = input_line in_channel in
+            if line_num = int_of_string line_num_str then
+               (* throw out any continued lines *)
+               process_in_lines lines [] (succ line_num)
+            else if Str.string_match cont_regex line 0 then
+               (* if there's a backslash, buffer the line in continuations list *)
+               process_in_lines lines (line :: continuations) (succ line_num)
+            else
+               (* if there's no backslash, go ahead and dump the continuations list
+                * into the list of lines to write *)
+               process_in_lines (line :: (continuations @ lines)) [] (succ line_num)
+         with End_of_file ->
+            close_in in_channel;
+            List.rev lines
+      in
+      let remaining_lines = process_in_lines [] [] 1 in
+      let out_channel = open_out filename in
+      (* write out the new file *)
+      let rec write_lines lines =
+         begin match lines with
+         | [] ->
+            close_out out_channel
+         | line :: tail ->
+            output_string out_channel (line ^ "\n");
+            write_lines tail
+         end
+      in
+      write_lines remaining_lines;
+      let r = Remind.create_three_month iface.top_timestamp in
+      (* if the untimed list has been altered, change the focus to
+       * the first element of the list *)
+      let new_iface =
+         if List.length r.Remind.curr_untimed <> 
+            List.length reminders.Remind.curr_untimed then {
+            iface with top_untimed = 0;
+                       top_desc = 0;
+                       right_selection = 1
+            }
+         else
+            iface
+      in
+      let final_iface, _ = handle_refresh new_iface r in
+      draw_error final_iface "cut reminder to clipboard." false;
+      assert (doupdate ());
+      (final_iface, r)
+   end
+
+
+(* handle jumping to a specified date *)
+let handle_goto (iface : interface_state_t) reminders =
+   let tm = Unix.localtime (Unix.time () -. (time_inc iface)) in
+   let len = String.length iface.goto_input in
+   if not (List.mem len [2; 4; 8]) then
+      failwith "length must be 2, 4, or 8 characters."
+   else begin
+      let (year_s, month_s, day_s) = 
+         if !Rcfile.goto_big_endian then
+            if len = 8 then (
+               String.sub iface.goto_input 0 4,
+               String.sub iface.goto_input 4 2,
+               String.sub iface.goto_input 6 2
+            )
+            else if len = 4 then (
+               string_of_int (tm.Unix.tm_year + 1900),
+               String.sub iface.goto_input 0 2,
+               String.sub iface.goto_input 2 2
+            )
+            else (
+               string_of_int (tm.Unix.tm_year + 1900),
+               string_of_int (tm.Unix.tm_mon + 1),
+               iface.goto_input
+            )
+         else
+            if len = 8 then (
+               String.sub iface.goto_input 4 4,
+               String.sub iface.goto_input 2 2,
+               String.sub iface.goto_input 0 2
+            )
+            else if len = 4 then (
+               string_of_int (tm.Unix.tm_year + 1900),
+               String.sub iface.goto_input 2 2,
+               String.sub iface.goto_input 0 2
+            )
+            else (
+               string_of_int (tm.Unix.tm_year + 1900),
+               string_of_int (tm.Unix.tm_mon + 1),
+               iface.goto_input
+            )
+      in
+      let year  = (int_of_string year_s) - 1900 
+      and month = (int_of_string month_s) - 1
+      and day   = int_of_string day_s in
+      let jump_time = {
+         tm with Unix.tm_year = year;
+                 Unix.tm_mon  = month;
+                 Unix.tm_mday = day
+      } in
+      let (rounded_time, rt) =
+         try
+            Unix.mktime (round_time iface.zoom_level jump_time)
+         with _ ->
+            failwith "requested date is out of range."
+      in
+      if rt.Unix.tm_mday <> jump_time.Unix.tm_mday then
+         failwith "requested day of the month is out of range."
+      else if rt.Unix.tm_mon <> jump_time.Unix.tm_mon then
+         failwith "requested month is out of range."
+      else if (rt.Unix.tm_year + 1900) < 1991 || (rt.Unix.tm_year + 1900) > 2074 then
+         failwith "requested year is out of range."
+      else
+         ();
+      let new_iface = {
+         iface with top_timestamp = 
+                       if !Rcfile.center_cursor then
+                          rounded_time -. (time_inc iface) *. (float_of_int ((iface.scr.tw_lines / 2) - 2))
+                       else
+                          rounded_time -. (time_inc iface) *. 1.;
+                    top_desc         = 0;
+                    selected_side    = Left;
+                    left_selection   = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 2;
+                    right_selection  = 1;
+                    is_entering_goto = false;
+                    goto_input       = ""
+      } in
+      handle_selection_change new_iface reminders
+   end
+         
+
+(* Begin entry of a date/time to navigate to *)
+let handle_begin_goto (iface : interface_state_t) reminders =
+   let new_iface = {
+      iface with is_entering_goto = true
+   } in
+   if !Rcfile.goto_big_endian then
+      draw_error iface "go to date [[YYYY]MM]DD: " true
+   else
+      draw_error iface "go to date DD[MM[YYYY]]: " true;
+   (new_iface, reminders)
+
+
+
 (* handle keyboard input and update the display appropriately *)
 let handle_keypress key (iface : interface_state_t) reminders =
-   if not iface.is_entering_search then begin
+   if not (iface.is_entering_search || iface.is_entering_goto) then begin
       try
          match Rcfile.command_of_key key with
          |Rcfile.ScrollDown ->
@@ -1086,14 +1432,55 @@ let handle_keypress key (iface : interface_state_t) reminders =
             handle_switch_focus iface reminders
          |Rcfile.Home ->
             handle_home iface reminders
+         |Rcfile.Goto ->
+            handle_begin_goto iface reminders
          |Rcfile.Edit ->
             handle_edit iface reminders
          |Rcfile.EditAny ->
-            let remfile = 
-               do_selection_dialog iface "Choose a reminders file"
-               (Remind.get_included_remfiles ())
+            let all_remfiles = Remind.get_included_remfiles () in
+            let selected_remfile = 
+               (* if there's only one remfile, jump right in, otherwise
+                * pop up a selection dialog *)
+               if List.length all_remfiles > 1 then
+                  do_selection_dialog iface "Choose a reminders file to edit"
+                     all_remfiles
+               else
+                  List.hd all_remfiles
             in
-            handle_edit_any iface reminders remfile
+            handle_edit_any iface reminders selected_remfile
+         |Rcfile.CopyReminder ->
+            handle_copy_reminder iface reminders true
+         |Rcfile.CutReminder ->
+            handle_cut_reminder iface reminders
+         |Rcfile.PasteReminder ->
+            if iface.rem_buffer = "" then begin
+               let _ = beep () in
+               draw_error iface "clipboard is empty." false;
+               assert (doupdate ());
+               (iface, reminders)
+            end else begin
+               handle_paste_reminder iface reminders 
+               (Utility.expand_file !Rcfile.reminders_file)
+            end
+         |Rcfile.PasteReminderDialog ->
+            if iface.rem_buffer = "" then begin
+               let _ = beep () in
+               draw_error iface "clipboard is empty." false;
+               assert (doupdate ());
+               (iface, reminders)
+            end else begin
+               let all_remfiles = Remind.get_included_remfiles () in
+               let selected_remfile = 
+                  (* if there's only one remfile, jump right in, otherwise
+                   * pop up a selection dialog *)
+                  if List.length all_remfiles > 1 then
+                     do_selection_dialog iface "Choose a reminders file to paste into"
+                        all_remfiles
+                  else
+                     List.hd all_remfiles
+               in
+               handle_paste_reminder iface reminders selected_remfile
+            end
          |Rcfile.ScrollDescUp ->
             handle_scroll_desc_up iface reminders
          |Rcfile.ScrollDescDown ->
@@ -1168,57 +1555,116 @@ let handle_keypress key (iface : interface_state_t) reminders =
       try
          begin match Rcfile.entry_of_key key with
          |Rcfile.EntryComplete ->
-            begin try
-               let new_iface = {
-                  iface with search_regex = Str.regexp_case_fold iface.search_input;
-                             search_input = "";
-                             is_entering_search = false
-               } in
-               handle_find_next new_iface reminders None
-            with Failure err ->
+            if iface.is_entering_search then
+               begin try
+                  let new_iface = {
+                     iface with search_regex = Str.regexp_case_fold iface.search_input;
+                                search_input = "";
+                                is_entering_search = false
+                  } in
+                  handle_find_next new_iface reminders None
+               with Failure err ->
+                  let new_iface = {
+                     iface with search_input = "";
+                                is_entering_search = false
+                  } in
+                  let _ = beep () in
+                  draw_error new_iface ("syntax error in search string: " ^ err) false;
+                  assert (doupdate ());
+                  (new_iface, reminders)
+               end
+            else
+               begin try
+                  handle_goto iface reminders
+               with Failure err ->
+                  let new_iface = {
+                     iface with is_entering_goto = false;
+                                goto_input = ""
+                  } in
+                  let _ = beep () in
+                  draw_error new_iface ("syntax error in date specifier: " ^ err) false;
+                  assert (doupdate ());
+                  (new_iface, reminders)
+               end
+         |Rcfile.EntryBackspace ->
+            if iface.is_entering_search then
+               let len = String.length iface.search_input in
+               if len > 0 then 
+                  let new_iface = {
+                     iface with search_input = 
+                                 Str.string_before iface.search_input (pred len)
+                  } in
+                  draw_error iface ("search expression: " ^ new_iface.search_input) true;
+                  (new_iface, reminders)
+               else
+                  let _ = beep () in
+                  (iface, reminders)
+            else
+               let len = String.length iface.goto_input in
+               if len > 0 then begin
+                  let new_iface = {
+                     iface with goto_input = 
+                                 Str.string_before iface.goto_input (pred len)
+                  } in
+                  if !Rcfile.goto_big_endian then
+                     draw_error iface ("go to date [[YYYY]MM]DD: " ^ new_iface.goto_input) true
+                  else
+                     draw_error iface ("go to date DD[MM[YYYY]]: " ^ new_iface.goto_input) true;
+                  (new_iface, reminders)
+               end else
+                  let _ = beep () in
+                  (iface, reminders)
+         |Rcfile.EntryExit ->
+            if iface.is_entering_search then begin
                let new_iface = {
                   iface with search_input = "";
                              is_entering_search = false
                } in
-               draw_error new_iface ("syntax error in search string: " ^ err) false;
-               assert (doupdate ());
+               draw_error new_iface "search cancelled." false;
+               (new_iface, reminders)
+            end else begin
+               let new_iface = {
+                  iface with goto_input = "";
+                             is_entering_goto = false
+               } in
+               draw_error new_iface "date entry cancelled." false;
                (new_iface, reminders)
             end
-         |Rcfile.EntryBackspace ->
-            let len = String.length iface.search_input in
-            if len > 0 then 
-               let new_iface = {
-               iface with search_input = 
-                           Str.string_before iface.search_input (pred len)
-               } in
-               draw_error iface ("search expression: " ^ new_iface.search_input) true;
-               (new_iface, reminders)
-            else
-               let _ = beep () in
-               (iface, reminders)
-         |Rcfile.EntryExit ->
-            let new_iface = {
-               iface with search_input = "";
-                          is_entering_search = false
-            } in
-            draw_error new_iface "search cancelled." false;
-            (new_iface, reminders)
          end
       with Not_found ->
-         begin try
-            (* only printable characters are accepted for search strings *)
-            if key >= 32 && key <= 126 then begin
-               let c = char_of_int key in
-               let new_iface = {
-                  iface with search_input = iface.search_input ^ (String.make 1 c)
-               } in
-               draw_error new_iface ("search expression: " ^ new_iface.search_input) true;
-               (new_iface, reminders)
-            end else
-               failwith "cannot search for unprintable characters"
-         with Failure _ ->
-            let _ = beep () in
-            (iface, reminders)
+         if iface.is_entering_search then begin 
+            try
+               (* only printable characters are accepted for search strings *)
+               if key >= 32 && key <= 126 then begin
+                  let c = char_of_int key in
+                  let new_iface = {
+                     iface with search_input = iface.search_input ^ (String.make 1 c)
+                  } in
+                  draw_error new_iface ("search expression: " ^ new_iface.search_input) true;
+                  (new_iface, reminders)
+               end else
+                  failwith "cannot search for unprintable characters"
+            with Failure _ ->
+               let _ = beep () in
+               (iface, reminders)
+         end else begin
+            try
+               (* only digits are accepted for goto dates *)
+               if key >= 48 && key <= 57 && (String.length iface.goto_input < 8) then begin
+                  let c = char_of_int key in
+                  let new_iface = {
+                     iface with goto_input = iface.goto_input ^ (String.make 1 c)
+                  } in
+                  if !Rcfile.goto_big_endian then
+                     draw_error iface ("go to date [[YYYY]MM]DD: " ^ new_iface.goto_input) true
+                  else
+                     draw_error iface ("go to date DD[MM[YYYY]]: " ^ new_iface.goto_input) true;
+                  (new_iface, reminders)
+               end else
+                  failwith "date characters must be digits"
+            with Failure _ ->
+               let _ = beep () in
+               (iface, reminders)
          end
    end
 
