@@ -45,7 +45,9 @@ type command_t = | ScrollUp | ScrollDown | NextDay | PrevDay
                  | SearchNext | BeginSearch | Quit | ViewReminders
                  | ScrollDescUp | ScrollDescDown | Refresh
                  | ViewAllReminders | ViewWeek | ViewMonth
-                 | NextReminder | ViewKeybindings
+                 | NextReminder | ViewKeybindings | CopyReminder
+                 | PasteReminder | PasteReminderDialog 
+                 | CutReminder | Goto
                  | NewGenReminder of int | NewGenReminderDialog of int
 
 type entry_operation_t = | EntryComplete | EntryBackspace | EntryExit
@@ -111,6 +113,8 @@ let status_12_hour      = ref true
 let description_12_hour = ref true
 (* Center the schedule on the cursor? *)
 let center_cursor = ref false
+(* "jump to" date syntax is big-endian? *)
+let goto_big_endian = ref true
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref []
 
@@ -236,7 +240,7 @@ let decode_single_key_string key_string =
       else 
          let octal_regex = Str.regexp "^0o" in
          try
-            let pos = Str.search_forward octal_regex key_string 0 in
+            let _ = Str.search_forward octal_regex key_string 0 in
             ((int_of_string key_string), ("\\" ^ Str.string_after key_string
             2))
          with
@@ -295,9 +299,14 @@ let commands_list = [
    ("next_month"              , CommandOp NextMonth);
    ("previous_month"          , CommandOp PrevMonth);
    ("home"                    , CommandOp Home);
+   ("goto"                    , CommandOp Goto);
    ("zoom"                    , CommandOp Zoom);
    ("edit"                    , CommandOp Edit);
    ("edit_any"                , CommandOp EditAny);
+   ("copy"                    , CommandOp CopyReminder);
+   ("cut"                     , CommandOp CutReminder);
+   ("paste"                   , CommandOp PasteReminder);
+   ("paste_dialog"            , CommandOp PasteReminderDialog);
    ("scroll_description_up"   , CommandOp ScrollDescUp);
    ("scroll_description_down" , CommandOp ScrollDescDown);
    ("new_timed"               , CommandOp NewTimed);
@@ -523,6 +532,8 @@ let parse_line line_stream =
          parse_set "description_12_hour" description_12_hour bool_of_string "Expected a boolean string after "
       | [< 'Ident "center_cursor" >] ->
          parse_set "center_cursor" center_cursor bool_of_string "Expected a boolean string after "
+      | [< 'Ident "goto_big_endian" >] ->
+         parse_set "goto_big_endian" goto_big_endian bool_of_string "Expected a boolean string after "
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
